/*
Jason Staggs 3/11/2013
University of Tulsa
Crash Reconstruction Research Consortium (TU-CRRC)
http://tucrrc.utulsa.edu/canclock
CAN Clock Project
2003 Mini Cooper S
*/

//importing CAN shield library
#include <MCP2515.h> 
#include <MCP2515_defs.h>
//Serial Perhiphial Interface
#include <SPI.h>
//RTC module library
#include <Wire.h>
#include "RTClib.h"


//Global variables
#define CLICK A4
//This defines the Slave Select pin for the CAN chip
#define CAN_CS 10
//interupt pin..
#define CAN_INT 2

MCP2515 CAN(CAN_CS,CAN_INT);

//Speedometer Frame object
Frame speedTx;
//Tachometer Frame object
Frame tachTx;

// CLOCK VARIABLES:
double hours = 0;
int minutes = 0;
int seconds = 0;

//clock boolean
boolean clock = true;

RTC_DS1307 RTC;

void setup()
{

Serial.begin(57600);
Wire.begin();
RTC.begin();

if(! RTC.isrunning())
{
 Serial.println("RTC is NOT running!"); 
}
//Sets the RTC to the date & time this sketch was compiled
    RTC.adjust(DateTime(__DATE__, __TIME__));
  
 Serial.begin(9600);
 Serial.println("Initializing...");
 
 SPI.setClockDivider(SPI_CLOCK_DIV2);
 SPI.setDataMode(SPI_MODE0);
 SPI.setBitOrder(MSBFIRST);
 SPI.begin();
 
 int baudRate=CAN.Init(500,16);
  if(baudRate>0) {
    Serial.println("MCP2515 Init OK ...");
    Serial.print("Baud Rate (kbps): ");
    Serial.println(baudRate,DEC);
  } else {
    Serial.println("MCP2515 Init Failed ...");
  }
  Serial.println("Initialization Complete.");
  
  //enable rollover...?
  CAN.BitModify(RXB0CTRL,0x04,0x04);

  pinMode(CLICK,INPUT);
  digitalWrite(CLICK, HIGH);
  
}

void loop()
{
  //clockMode();
  if(clock)
  {
    clockMode();
  }
  
  if(digitalRead(CLICK) == 0 || clock == false)
  {
    clock = false;
    demoMode();
  }
  
  if(clock == false && digitalRead(CLICK) == 0)
 {
  clock = true;
 } 
  
}

void clockMode()
{
  DateTime now = RTC.now();
  hours = now.hour();
  minutes = now.minute();
  seconds = now.second();
  
  if(hours > 12)
  {
   hours = hours - 12; 
  }
  if(hours == 0)
  {
   hours = 12; 
  }
  
 
  //fun action for hours
  if(minutes == 00 && seconds < 3)
  {
   hours = 15; 
  }
  //Scale from program value to dial value
  hours = (hours + ((double)minutes / 60)) * 7.9;
  
    //fun action for minutes
    //scale minutes
  if(seconds < 3)
  {
    minutes = 80;
  }
  minutes = minutes *2.5; 
  
 
  
  speedTx.id = 0x153;
  speedTx.srr = 0x0;
  speedTx.rtr = 0x0;
  speedTx.dlc = 0x8;
  speedTx.data[2] = hours;
  //Sending the frame on its way
  CAN.LoadBuffer(TXB0,speedTx);
  CAN.SendBuffer(TXB0);
  delay(5);
  
  tachTx.id = 0x316;
  tachTx.srr = 0x0;
  tachTx.rtr = 0x0;
  tachTx.dlc = 0x8;
  tachTx.data[3] = minutes;
  
  //Sending the frame on its way
  CAN.LoadBuffer(TXB0,tachTx);
  CAN.SendBuffer(TXB0);
  delay(80);
}

void demoMode()
{
 speedTx.id = 0x153;
  speedTx.srr = 0x0;
  speedTx.rtr = 0x0;
  speedTx.dlc = 0x8;
  speedTx.data[2] = hours;
  //Sending the frame on its way
  CAN.LoadBuffer(TXB0,speedTx);
  CAN.SendBuffer(TXB0);
  hours++;
  if(hours > 95)
  {
    hours = 0;
  }
  
  delay(5);
  
  tachTx.id = 0x316;
  tachTx.srr = 0x0;
  tachTx.rtr = 0x0;
  tachTx.dlc = 0x8;
  tachTx.data[3] = minutes;
  
  //Sending the frame on its way
  CAN.LoadBuffer(TXB0,tachTx);
  CAN.SendBuffer(TXB0);
  minutes++;
  delay(5);
  
  if(minutes > 150)
  {
    minutes = 0;
  }
  delay(80); 
}
